"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.exit = exports.askForMissingInformation = exports.NewAction = void 0;
const strings_1 = require("@angular-devkit/core/src/utils/strings");
const chalk = require("chalk");
const fs = require("fs");
const inquirer = require("inquirer");
const path_1 = require("path");
const util_1 = require("util");
const defaults_1 = require("../lib/configuration/defaults");
const package_managers_1 = require("../lib/package-managers");
const questions_1 = require("../lib/questions/questions");
const git_runner_1 = require("../lib/runners/git.runner");
const schematics_1 = require("../lib/schematics");
const ui_1 = require("../lib/ui");
const abstract_action_1 = require("./abstract.action");
const project_utils_1 = require("../lib/utils/project-utils");
const auth_guard_1 = require("../lib/utils/auth-guard");
const api_service_1 = require("../lib/apiService/api.service");
const regex_1 = require("../lib/utils/regex");
class NewAction extends abstract_action_1.AbstractAction {
    handle(inputs, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const directoryOption = options.find(option => option.name === 'directory');
            const dryRunOption = options.find(option => option.name === 'dry-run');
            const isDryRunEnabled = dryRunOption && dryRunOption.value;
            console.info(ui_1.MESSAGES.PROJECT_INFORMATION_START);
            console.info();
            yield auth_guard_1.authGuard(inputs);
            yield exports.askForMissingInformation(inputs);
            yield project_utils_1.selectMemory(inputs);
            yield project_utils_1.selectReplicas(inputs);
            yield project_utils_1.selectRuntime(inputs);
            yield selectPackageManager(options);
            yield generateApplicationFiles(inputs, options).catch(exports.exit);
            const shouldSkipInstall = options.some(option => option.name === 'skip-install' && option.value === true);
            const shouldSkipGit = options.some(option => option.name === 'skip-git' && option.value === true);
            const projectDirectory = getProjectDirectory(project_utils_1.getApplicationNameInput(inputs), directoryOption);
            if (!shouldSkipInstall) {
                yield installPackages(options, isDryRunEnabled, projectDirectory);
            }
            if (!isDryRunEnabled) {
                if (!shouldSkipGit) {
                    yield initializeGitRepository(projectDirectory);
                    yield createGitIgnoreFile(projectDirectory);
                }
                printCollective();
            }
            process.exit(0);
        });
    }
}
exports.NewAction = NewAction;
const getProjectDirectory = (applicationName, directoryOption) => {
    return ((directoryOption && directoryOption.value) ||
        strings_1.dasherize(applicationName.value));
};
const generateApplicationFiles = (args, options) => __awaiter(void 0, void 0, void 0, function* () {
    const collectionName = options.find(option => option.name === 'collection' && option.value != null).value;
    const collection = schematics_1.CollectionFactory.create(collectionName || schematics_1.Collection.MICROFUNCTION);
    const schematicOptions = mapSchematicOptions(args.concat(options));
    yield collection.execute('functions', schematicOptions);
    console.info();
});
const mapSchematicOptions = (options) => {
    return options.reduce((schematicOptions, option) => {
        if (option.name !== 'skip-install' &&
            option.value !== 'packageManager') {
            schematicOptions.push(new schematics_1.SchematicOption(option.name, option.value));
        }
        return schematicOptions;
    }, []);
};
const installPackages = (options, dryRunMode, installDirectory) => __awaiter(void 0, void 0, void 0, function* () {
    const inputPackageManager = options.find(option => option.name === 'packageManager').value;
    let packageManager;
    if (dryRunMode) {
        console.info();
        console.info(chalk.green(ui_1.MESSAGES.DRY_RUN_MODE));
        console.info();
        return;
    }
    try {
        packageManager = package_managers_1.PackageManagerFactory.create(inputPackageManager);
        yield packageManager.install(installDirectory, inputPackageManager);
    }
    catch (error) {
        if (error && error.message) {
            console.error(chalk.red(error.message));
        }
    }
});
const selectPackageManager = (inputs) => __awaiter(void 0, void 0, void 0, function* () {
    const answers = yield askForPackageManager();
    project_utils_1.replaceInputMissingInformation(inputs, answers);
    return package_managers_1.PackageManagerFactory.create(answers['packageManager']);
});
const askForPackageManager = () => __awaiter(void 0, void 0, void 0, function* () {
    const questions = [
        questions_1.generateSelect('packageManager')(ui_1.MESSAGES.PACKAGE_MANAGER_QUESTION)([
            package_managers_1.PackageManager.NPM,
            package_managers_1.PackageManager.YARN,
        ]),
    ];
    const prompt = inquirer.createPromptModule();
    return yield prompt(questions);
});
const initializeGitRepository = (dir) => __awaiter(void 0, void 0, void 0, function* () {
    const runner = new git_runner_1.GitRunner();
    yield runner.run('init', true, path_1.join(process.cwd(), dir)).catch(() => {
        console.error(chalk.red(ui_1.MESSAGES.GIT_INITIALIZATION_ERROR));
    });
});
/**
 * Write a file `.gitignore` in the root of the newly created project.
 * `.gitignore` available in `@nestjs/schematics` cannot be published to
 * NPM (needs to be investigated).
 *
 * @param dir Relative path to the project.
 * @param content (optional) Content written in the `.gitignore`.
 *
 * @return Resolves when succeeds, or rejects with any error from `fn.writeFile`.
 */
const createGitIgnoreFile = (dir, content) => {
    const fileContent = content || defaults_1.defaultGitIgnore;
    const filePath = path_1.join(process.cwd(), dir, '.gitignore');
    return util_1.promisify(fs.writeFile)(filePath, fileContent);
};
const printCollective = () => {
    const emptyLine = project_utils_1.print();
    emptyLine();
    const dim = project_utils_1.print('dim');
    const yellow = project_utils_1.print('yellow');
    emptyLine();
    yellow(`Thanks for installing @microfunctions/cli ${ui_1.EMOJIS.PRAY}`);
    project_utils_1.print()(`${chalk.bold(`${ui_1.EMOJIS.WINE}  blog:`)} ${chalk.underline('https://microfunctions.io/blog')}`);
    emptyLine();
};
exports.askForMissingInformation = (inputs) => __awaiter(void 0, void 0, void 0, function* () {
    const prompt = inquirer.createPromptModule();
    const functions = project_utils_1.getInputsValue(inputs, 'functions');
    const namespaceInput = project_utils_1.getApplicationNamespaceInput(inputs);
    const executedNameInput = project_utils_1.getApplicationExecutedNameInput(inputs);
    if (!functions) {
        const message = 'What name would you like to use for the new function ?';
        const questions = [questions_1.generateInput('functions', message)('microFunction')];
        const answers = yield prompt(questions);
        if (answers['functions'].match(regex_1.regexNameFunction) == null) {
            console.error(`${answers['functions']} is invalid regex used for validation is '/[a-z0-9]([-a-z0-9]*[a-z0-9])?(\\.[a-z0-9]([-a-z0-9]*[a-z0-9])?)*$/' `);
            yield exports.askForMissingInformation(inputs);
        }
        project_utils_1.replaceInputMissingInformation(inputs, answers);
    }
    if (!namespaceInput.value) {
        let answers;
        const namespaces = yield api_service_1.getAllNamespaces(inputs);
        if (namespaces.length > 0) {
            answers = yield askForNamespaces(namespaces);
            const namespace = namespaces.find((namespace) => namespace.name === answers['namespace']);
            answers['namespace'] = namespace.name;
            answers['idNamespace'] = namespace.id;
        }
        else {
            console.error(chalk.red('Please add a namespace from console'));
            project_utils_1.print()(`${chalk.underline('https://console.microfunctions.io/microfunction/namespace/create')}`);
            exports.exit();
        }
        project_utils_1.replaceInputMissingInformation(inputs, answers);
    }
    if (!executedNameInput.value) {
        const message = 'What ExecutedName would you like to use ?';
        const questions = [questions_1.generateInput('executedName', message)('helloword')];
        const answers = yield prompt(questions);
        if (!regex_1.isValidFnName(answers['executedName'])) {
            console.error(`${answers['executedName']} is invalid regex used for validation is '/[a-z0-9]([-a-z0-9]*[a-z0-9])?(\\.[a-z0-9]([-a-z0-9]*[a-z0-9])?)*$/' `);
            yield exports.askForMissingInformation(inputs);
        }
        project_utils_1.replaceInputMissingInformation(inputs, answers);
    }
});
const askForNamespaces = (namespaces) => __awaiter(void 0, void 0, void 0, function* () {
    const namespacesAsk = namespaces.map((namespace) => namespace.name);
    const questions = [
        questions_1.generateSelect('namespace')(ui_1.MESSAGES.NAMESPACE_QUESTION)(namespacesAsk),
    ];
    const prompt = inquirer.createPromptModule();
    return yield prompt(questions);
});
exports.exit = () => process.exit(1);
